from pyconnectwise.models.base.message_model import GenericMessageModel
from pyconnectwise.endpoints.base.connectwise_endpoint import ConnectWiseEndpoint
from pyconnectwise.responses.paginated_response import PaginatedResponse
from typing import Any
from pyconnectwise.endpoints.manage.CompanyCompaniesIdCustomStatusNotesIdEndpoint import CompanyCompaniesIdCustomStatusNotesIdEndpoint
from pyconnectwise.endpoints.manage.CompanyCompaniesIdCustomStatusNotesCountEndpoint import CompanyCompaniesIdCustomStatusNotesCountEndpoint
from pyconnectwise.models.manage.CompanyCustomNoteModel import CompanyCustomNoteModel

class CompanyCompaniesIdCustomStatusNotesEndpoint(ConnectWiseEndpoint):
    def __init__(self, client, parent_endpoint=None):
        super().__init__(client, "customStatusNotes", parent_endpoint=parent_endpoint)
        
        self.count = self.register_child_endpoint(
            CompanyCompaniesIdCustomStatusNotesCountEndpoint(client, parent_endpoint=self)
        )
    
    
    def id(self, id: int) -> CompanyCompaniesIdCustomStatusNotesIdEndpoint:
        """
        Sets the ID for this endpoint and returns an initialized CompanyCompaniesIdCustomStatusNotesIdEndpoint object to move down the chain.

        Parameters:
            id (int): The ID to set.
        Returns:
            CompanyCompaniesIdCustomStatusNotesIdEndpoint: The initialized CompanyCompaniesIdCustomStatusNotesIdEndpoint object.
        """
        child = CompanyCompaniesIdCustomStatusNotesIdEndpoint(self.client, parent_endpoint=self)
        child._id = id
        return child
    
    def paginated(self, page: int, page_size: int, params: dict[str, int | str] = {}) -> PaginatedResponse[CompanyCustomNoteModel]:
        """
        Performs a GET request against the /company/companies/{parentId}/customStatusNotes endpoint and returns an initialized PaginatedResponse object.

        Parameters:
            page (int): The page number to request.
            page_size (int): The number of results to return per page.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            PaginatedResponse[CompanyCustomNoteModel]: The initialized PaginatedResponse object.
        """
        params["page"] = page
        params["pageSize"] = page_size
        return PaginatedResponse(
            super().make_request(
                "GET",
                params=params
            ),
            CompanyCustomNoteModel,
            self,
            page_size,
        )
    
    def get(self, data: dict[str, Any] = {}, params: dict[str, int | str] = {}) -> list[CompanyCustomNoteModel]:
        """
        Performs a GET request against the /company/companies/{parentId}/customStatusNotes endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            list[CompanyCustomNoteModel]: The parsed response data.
        """
        return self._parse_many(CompanyCustomNoteModel, super().make_request("GET", data=data, params=params).json())
        
    def post(self, data: dict[str, Any] = {}, params: dict[str, int | str] = {}) -> CompanyCustomNoteModel:
        """
        Performs a POST request against the /company/companies/{parentId}/customStatusNotes endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            CompanyCustomNoteModel: The parsed response data.
        """
        return self._parse_one(CompanyCustomNoteModel, super().make_request("POST", data=data, params=params).json())
        