from pyconnectwise.models.base.message_model import GenericMessageModel
from pyconnectwise.endpoints.base.connectwise_endpoint import ConnectWiseEndpoint
from pyconnectwise.responses.paginated_response import PaginatedResponse
from typing import Any
from pyconnectwise.endpoints.manage.MarketingCampaignsIdFormsSubmittedIdEndpoint import MarketingCampaignsIdFormsSubmittedIdEndpoint
from pyconnectwise.endpoints.manage.MarketingCampaignsIdFormsSubmittedCountEndpoint import MarketingCampaignsIdFormsSubmittedCountEndpoint
from pyconnectwise.models.manage.FormSubmittedModel import FormSubmittedModel

class MarketingCampaignsIdFormsSubmittedEndpoint(ConnectWiseEndpoint):
    def __init__(self, client, parent_endpoint=None):
        super().__init__(client, "formsSubmitted", parent_endpoint=parent_endpoint)
        
        self.count = self.register_child_endpoint(
            MarketingCampaignsIdFormsSubmittedCountEndpoint(client, parent_endpoint=self)
        )
    
    
    def id(self, id: int) -> MarketingCampaignsIdFormsSubmittedIdEndpoint:
        """
        Sets the ID for this endpoint and returns an initialized MarketingCampaignsIdFormsSubmittedIdEndpoint object to move down the chain.

        Parameters:
            id (int): The ID to set.
        Returns:
            MarketingCampaignsIdFormsSubmittedIdEndpoint: The initialized MarketingCampaignsIdFormsSubmittedIdEndpoint object.
        """
        child = MarketingCampaignsIdFormsSubmittedIdEndpoint(self.client, parent_endpoint=self)
        child._id = id
        return child
    
    def paginated(self, page: int, page_size: int, params: dict[str, int | str] = {}) -> PaginatedResponse[FormSubmittedModel]:
        """
        Performs a GET request against the /marketing/campaigns/{parentId}/formsSubmitted endpoint and returns an initialized PaginatedResponse object.

        Parameters:
            page (int): The page number to request.
            page_size (int): The number of results to return per page.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            PaginatedResponse[FormSubmittedModel]: The initialized PaginatedResponse object.
        """
        params["page"] = page
        params["pageSize"] = page_size
        return PaginatedResponse(
            super().make_request(
                "GET",
                params=params
            ),
            FormSubmittedModel,
            self,
            page_size,
        )
    
    def get(self, data: dict[str, Any] = {}, params: dict[str, int | str] = {}) -> list[FormSubmittedModel]:
        """
        Performs a GET request against the /marketing/campaigns/{parentId}/formsSubmitted endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            list[FormSubmittedModel]: The parsed response data.
        """
        return self._parse_many(FormSubmittedModel, super().make_request("GET", data=data, params=params).json())
        
    def post(self, data: dict[str, Any] = {}, params: dict[str, int | str] = {}) -> FormSubmittedModel:
        """
        Performs a POST request against the /marketing/campaigns/{parentId}/formsSubmitted endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            FormSubmittedModel: The parsed response data.
        """
        return self._parse_one(FormSubmittedModel, super().make_request("POST", data=data, params=params).json())
        