from pyconnectwise.models.base.message_model import GenericMessageModel
from pyconnectwise.endpoints.base.connectwise_endpoint import ConnectWiseEndpoint
from pyconnectwise.responses.paginated_response import PaginatedResponse
from typing import Any
from pyconnectwise.endpoints.manage.SystemConnectWiseHostedScreensIdEndpoint import SystemConnectWiseHostedScreensIdEndpoint
from pyconnectwise.endpoints.manage.SystemConnectWiseHostedScreensCountEndpoint import SystemConnectWiseHostedScreensCountEndpoint
from pyconnectwise.models.manage.ConnectWiseHostedScreenModel import ConnectWiseHostedScreenModel

class SystemConnectWiseHostedScreensEndpoint(ConnectWiseEndpoint):
    def __init__(self, client, parent_endpoint=None):
        super().__init__(client, "connectWiseHostedScreens", parent_endpoint=parent_endpoint)
        
        self.count = self.register_child_endpoint(
            SystemConnectWiseHostedScreensCountEndpoint(client, parent_endpoint=self)
        )
    
    
    def id(self, id: int) -> SystemConnectWiseHostedScreensIdEndpoint:
        """
        Sets the ID for this endpoint and returns an initialized SystemConnectWiseHostedScreensIdEndpoint object to move down the chain.

        Parameters:
            id (int): The ID to set.
        Returns:
            SystemConnectWiseHostedScreensIdEndpoint: The initialized SystemConnectWiseHostedScreensIdEndpoint object.
        """
        child = SystemConnectWiseHostedScreensIdEndpoint(self.client, parent_endpoint=self)
        child._id = id
        return child
    
    def paginated(self, page: int, page_size: int, params: dict[str, int | str] = {}) -> PaginatedResponse[ConnectWiseHostedScreenModel]:
        """
        Performs a GET request against the /system/connectWiseHostedScreens endpoint and returns an initialized PaginatedResponse object.

        Parameters:
            page (int): The page number to request.
            page_size (int): The number of results to return per page.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            PaginatedResponse[ConnectWiseHostedScreenModel]: The initialized PaginatedResponse object.
        """
        params["page"] = page
        params["pageSize"] = page_size
        return PaginatedResponse(
            super().make_request(
                "GET",
                params=params
            ),
            ConnectWiseHostedScreenModel,
            self,
            page_size,
        )
    
    def get(self, data: dict[str, Any] = {}, params: dict[str, int | str] = {}) -> list[ConnectWiseHostedScreenModel]:
        """
        Performs a GET request against the /system/connectWiseHostedScreens endpoint.

        Parameters:
            data (dict[str, Any]): The data to send in the request body.
            params (dict[str, int | str]): The parameters to send in the request query string.
        Returns:
            list[ConnectWiseHostedScreenModel]: The parsed response data.
        """
        return self._parse_many(ConnectWiseHostedScreenModel, super().make_request("GET", data=data, params=params).json())
        